
/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy80800.c
   \date 2006-10-01
   \brief Interface implementation for DANUBE board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy80800.h"
#include "common_fxo.h"
#include "pcm.h"

#ifdef TERIDIAN_FXO
   #include "teridian_fxo.h"
#endif 

#include "asm/danube/port.h"

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vmmc11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vmmc11 and data will send actions to
    /dev/vmmc12.
    -1 means no channel at startup */

/** Max phones on VMMC (analog channels)
    It is used only for STARTUP_MAPPING_VMMC[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST */
enum { MAX_PHONES_VMMC = 2 };

/** Max Teridian channels on VMMC */
enum { MAX_TERIDIAN_CH_VMMC = 1 };

/** Number of PCM highways */
enum { PCM_HIGHWAYS = 1 };

/** Name of board */
static const char* BOARD_NAME_EASY80800 = "EASY80800";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_VMMC[MAX_PHONES_VMMC] =
{
   /** phone, data, pcm */
   {  0,  0,  0 },
   {  1,  1,  1 }
};

/** Handle to the GPIO port */
static IFX_int32_t GPIO_PortFd = -1;

/** GPIO port name */
static IFX_char_t* GPIO_PORT_NAME = "/dev/danube-port";

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy80800_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy80800_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy80800_CfgPCMInterface(BOARD_t* pBoard,
                                             IFX_boolean_t hMaster);
IFX_return_t BOARD_Easy80800_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath,
                                             IFX_int32_t *pPhoneChCnt,
                                             IFX_int32_t *pDataChCnt,
                                             IFX_int32_t *pPCM_ChCnt);
IFX_return_t BOARD_Easy80800_OpenGPIO_Port(IFX_void_t);
IFX_return_t BOARD_Easy80800_CloseGPIO_Port(IFX_void_t);
IFX_void_t BOARD_Easy80800_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy80800_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);


/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy80800_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;
   TAPIDEMO_DEVICE_FXO_t* pFXODevice = IFX_NULL;
   IFX_return_t ret;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First three variables (id, type, sub type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   pBoard->nUseSys_FD = IFX_FALSE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &VmmcDevInitStruct;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_VMMC;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_VMMC;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_VMMC_Old;
   pCPUDevice->pszBBD_CRAM_File = sBBD_CRAM_File_VMMC;
#ifdef ABACUS_TEST
   pBoard->fBBD_Changed = IFX_FALSE;
#endif /* ABACUS_TEST */
#ifndef VXWORKS
   /* if using default download path and .tapidemorc file exists
      then change downloads path to specified in .tapidemorc */
   ret = Common_CheckDownloadPath(pPath, pBoard, pCPUDevice);
   if (IFX_SUCCESS != ret) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("Using default download path: %s.\n", 
             DEF_DOWNLOAD_PATH));
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      free(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* allocate memory for FXO device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for FXO Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* Init structure for FXO device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_FXO;
   pFXODevice = &pDevice->uDevice.stFxo;
   pFXODevice->nFxoType = FXO_TYPE_TERIDIAN;
   pFXODevice->nIrqNum = INT_NUM_IM2_IRL31;
   pFXODevice->nCountry = TERIDIAN_COUNTRY;
   pFXODevice->Reset = BOARD_Easy80800_Reset;
   pFXODevice->SetPin_Direction_onGPIO = BOARD_Easy80800_SetPin_Direction_onGPIO;
   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      free(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, FXO device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (DEVICE_Vmmc_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

#ifdef TERIDIAN_FXO
   if (TERIDIAN_Register(pFXODevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, FXO device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#endif

   if (BOARD_Easy80800_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_VMMC;

   pBoard->nMaxFxoCh = MAX_TERIDIAN_CH_VMMC;
   if (IFX_ERROR == BOARD_Easy80800_GetCapabilities(pBoard, pPath,
                                                    &pBoard->nMaxAnalogCh,
                                                    &pBoard->nMaxCoderCh,
                                                    &pBoard->nMaxPCM_Ch))
   {
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->nUsedChannels = pBoard->nMaxCoderCh;
   }
   else if (pBoard->nMaxAnalogCh > 0)
   {
      /* use max analog ch, which must be != 0, if also 0 then ERROR */
      pBoard->nUsedChannels = pBoard->nMaxAnalogCh;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Analog channels are missing! (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      pBoard->nCh_FD = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   } /* if (pBoard->nMaxCoderCh > 0) */

   pBoard->rgoPhones = calloc(pBoard->nMaxAnalogCh, sizeof(PHONE_t));
   pBoard->rgoFXOs = calloc(pBoard->nMaxFxoCh, sizeof(FXO_t));

   if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->nCh_FD == IFX_NULL)
       || (pBoard->rgnSockets == IFX_NULL) || (pBoard->rgoPhones == IFX_NULL)
       || (pBoard->rgoFXOs == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory to prepare board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
      pBoard->nCh_FD[i] = -1;
      pBoard->rgnSockets[i] = -1;
   }

   /* Teridian */
   pBoard->nFxoCh_FD = calloc(pBoard->nMaxFxoCh, sizeof(IFX_int32_t));

   if (pBoard->nFxoCh_FD == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory to fxo channels. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxFxoCh; i++)
   {
       pBoard->nFxoCh_FD[i] = -1;
   }

   /* Prepare array of free PCM channels */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for PCM channels. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   /* Store chip count */
   pBoard->nChipCnt = 1;

   /* Sets file descriptors */
   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   BOARD_Easy80800_OpenGPIO_Port();

#ifdef ABACUS_TEST
   pBoard->pChCustomMapping = IFX_NULL;
   if (nVerifySystemInitializtion)
   {
      if (IFX_ERROR == COM_PrepareVerifySystemInit(pBoard->pCtrl))
      {
         return IFX_ERROR;
      }
   }
#endif /* ABACUS_TEST */

   return IFX_SUCCESS;
}

/**
   Prepare device/channels names.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy80800_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_VMMC, strlen(sCH_DEV_NAME_VMMC));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_VMMC, 
           strlen(sCTRL_DEV_NAME_VMMC));
   snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   snprintf(tmp_buf, 2, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);

   /** System device name, like /dev/easy3332/[board id], ... */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_VMMC, 
           strlen(sSYS_DEV_NAME_VMMC));
   snprintf(tmp_buf, 1, "%d", (int) pBoard->nID - 1);
   strncat(pBoard->pszSystemDevName, tmp_buf, 1);
#ifdef FXO
   /** teridian device name without numbers, like /dev/ter */
   strncpy(pBoard->pszFxoChDevName, sTERIDIAN_CH_DEV_NAME, 
           strlen(sTERIDIAN_CH_DEV_NAME));

   /** Device control name numbers, like /dev/ter[board id]0, ... */
   strncpy(pBoard->pszFxoCtrlDevName, sTERIDIAN_CTRL_DEV_NAME, 
           strlen(sTERIDIAN_CTRL_DEV_NAME));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
#endif /* FXO */
   return IFX_SUCCESS;
}

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download
   \param pPhoneChCnt - pointer to phone channels count
   \param pDataChCnt  - pointer to data channels count
   \param pPCM_ChCnt  - pointer to pcm channels count

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy80800_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath,
                                             IFX_int32_t *pPhoneChCnt,
                                             IFX_int32_t *pDataChCnt,
                                             IFX_int32_t *pPCM_ChCnt)
{
   IFX_return_t ret = IFX_ERROR;

   if ((IFX_NULL == pPhoneChCnt) || (IFX_NULL == pDataChCnt) ||
       (IFX_NULL == pPCM_ChCnt) || (pBoard == IFX_NULL) ||
       (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
   if (0 >= pBoard->nDevCtrl_FD)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid device control file descriptor %s.\n"
            "(File: %s, line: %d)\n",
            pBoard->pszCtrlDevName, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = DEVICE_Vmmc_setupChannel(pBoard, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Channel 0 is not initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      close(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard, pPhoneChCnt, pDataChCnt, pPCM_ChCnt);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   close(pBoard->nDevCtrl_FD);
   pBoard->nDevCtrl_FD = -1;
   if (*pPhoneChCnt > MAX_PHONES_VMMC)
   {
      /* If MAX_PHONES_VMMC is too small than STARTUP_MAPPING_VMMC[] table 
         has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_VMMC is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}

#ifndef STREAM_1_1
/**
   Configure PCM interface, can define clock, master/slave mode, ...

   \param pBoard  - pointer to board
   \param hMaster  - IFX_TRUE - master mode, IFX_FALSE - slave mode

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark Should be also used to enable clock if extension board will be used.
*/
IFX_return_t BOARD_Easy80800_CfgPCMInterface(BOARD_t* pBoard,
                                             IFX_boolean_t hMaster)
{
   IFX_int32_t j;
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));
   if (hMaster)
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_MASTER;
   }
   else
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE_AUTOFREQ;
   }
   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
        ("%s: Configure PCM interface as %s, %s kHz\n",
         pBoard->pszBoardName,
         (hMaster ? "master":"slave"), TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));
   
   
   for (j = 0; j < PCM_HIGHWAYS; j++)
   {
      TD_ASSERT(IFX_SUCCESS == (ret = ioctl(pBoard->nDevCtrl_FD,
         IFX_TAPI_PCM_IF_CFG_SET, (IFX_int32_t) &pcm_if)), pBoard->nDevCtrl_FD,
         pBoard, 0, 0);
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
              ("Err configure PCM interface "
               "(File: %s, line: %d)\n", 
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* for */

   /* set PCM frequency */
   oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq;

   return IFX_SUCCESS;
} /* BOARD_Easy50712_CfgPCMInterface() */

#else /* !STREAM_1_1 */

IFX_return_t BOARD_Easy80800_CfgPCMInterface(BOARD_t* pBoard,
                                             IFX_boolean_t hMaster)
{
   return IFX_SUCCESS;
} /* PCM_Init() */
#endif /* !STREAM_1_1 */

/**
   Open GPIO port.

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy80800_OpenGPIO_Port(IFX_void_t)
{
   /* Device: mknod /dev/danube-port c 249 0, but look
      cat /proc/devices for correct major number. */
   if (GPIO_PortFd < 0)
   {
      GPIO_PortFd = open(GPIO_PORT_NAME, O_RDWR);

      if (GPIO_PortFd < 0)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err open GPIO port (%s), (File: %s, line: %d)\n",
                GPIO_PORT_NAME, __FILE__, __LINE__));
         return IFX_ERROR;
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("GPIO port (%s) opened\n",
                                            GPIO_PORT_NAME));
      }
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("GPIO port (%s) already opened\n",
                                         GPIO_PORT_NAME));
   }

   return IFX_SUCCESS;
}


/**
   Close GPIO port.

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy80800_CloseGPIO_Port(IFX_void_t)
{
   /* Device: mknod /dev/danube-port c 249 0, but look
      cat /proc/devices for correct major number. */
   if (GPIO_PortFd > 0)
   {
      if (close(GPIO_PortFd) == IFX_ERROR)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err closing GPIO port %s, (File: %s, line: %d)\n",
                 GPIO_PORT_NAME, __FILE__, __LINE__));
         return IFX_ERROR;
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("GPIO port (%s) closed\n",
                                            GPIO_PORT_NAME));
         GPIO_PortFd = -1;
      }
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("GPIO port (%s) already closed\n",
                                         GPIO_PORT_NAME));
   }

   return IFX_SUCCESS;
}

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy80800_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      free(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      free(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->rgoFXOs != IFX_NULL)
   {
      free(pBoard->rgoFXOs);
      pBoard->rgoFXOs = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      free(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      free(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->nFxoCh_FD != IFX_NULL)
   {
      free(pBoard->nFxoCh_FD);
      pBoard->nFxoCh_FD = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      free(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
   BOARD_Easy80800_CloseGPIO_Port();
}

/**
   Init board.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Easy80800_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
    /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy80800_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_Vmmc_Init(pBoard, pProgramArg->sPathToDwnldFiles) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize the PCM Interface. Must do this, otherwise access to 
      extension board or FXO w will not work. */
   if (BOARD_Easy80800_CfgPCMInterface(pBoard, 
                                       pProgramArg->oArgFlags.nPCM_Master) == IFX_ERROR)
   {
       return IFX_ERROR;
   }
#ifdef FXO
   /* FXO */
   if (pProgramArg->oArgFlags.nFXO)
   {
      if (IFX_SUCCESS != Common_Set_Fxo_FDs(pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error setting FXO fds. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         Common_Close_Fxo_FDs(pBoard);
         return IFX_ERROR;
      }

      if (IFX_SUCCESS != FXO_Init(pBoard, GPIO_TERIDIAN_EASY80800, FXO_TYPE_TERIDIAN))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error FXO initialization. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         Common_Close_Fxo_FDs(pBoard);
         return IFX_ERROR;
      }
   }
#endif /* FXO */

   return IFX_SUCCESS;
}

/**
   Set direction of GPIO pin to out which is used for set/clear reset.

   \param eType - devices to which access is done by GPIO

   \return IFX_SUCCESS if okm otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy80800_SetPin_Direction_onGPIO(GPIO_ACCESS_DEV_t eType)
{
   /* Reset of Teridian is not needed by application. It is performed 
      by TAPI_INIT. */
   return IFX_SUCCESS;   
};


/*
   Set/Clear reset of device by using GPIO port.

   \param pBoard - pointer to Board
   \param nArg - devices to which access is done by GPIO
   \param hSetReset - IFX_TRUE - set reset, IFX_FALSE - clear reset

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
 */
IFX_return_t BOARD_Easy80800_Reset(BOARD_t* pBoard,
                                   IFX_int32_t nArg,
                                   IFX_boolean_t hSetReset)
{
   /* Reset of Teridian is not needed by application. It is performed 
      by TAPI_INIT. */
   return IFX_SUCCESS;
}

/**
   Set PCM master mode.

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy80800_SetPCM_Master_overGPIO(IFX_void_t)
{
   struct danube_port_ioctl_parm parm = {0};
   IFX_return_t ret = IFX_SUCCESS;

   /* tsc */
   parm.pin  = 8;
   parm.port = 1;
   parm.value = 0;
   ret = ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL0, &parm);
   if (OK != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err clear alternate select register0 bit, P1.8 0x%0X, (File: %s, line: %d)\n",
             ret, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   parm.value = 1;
   ret = ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL1, &parm);
   if (OK != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err set alternate select register1 bit, P1.8 0x%0X, (File: %s, line: %d)\n",
             ret, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   parm.value = 1;
   ret = ioctl(GPIO_PortFd, DANUBE_PORT_IOCDIR, &parm);
   if (OK != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err set direction to output, P1.8 0x%0X, (File: %s, line: %d)\n",
             ret, __FILE__, __LINE__));
      return IFX_ERROR;
   }

#if 0
   /* do */
   parm.pin  = 9;
   parm.port = 1;
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL0, &parm);
   parm.value = 0;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL1, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCDIR, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCOD, &parm);

   /* di */
   parm.pin  = 10;
   parm.port = 1;
   parm.value = 0;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL0, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL1, &parm);
   parm.value = 0;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCDIR, &parm);

   /* dcl as */
   parm.pin  = 11;
   parm.port = 1;
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL0, &parm);
   parm.value = 0;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL1, &parm);
   parm.value = 1; /* dcl used as output */
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCDIR, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCOD, &parm);

   /* fsc */
   parm.pin = 0;
   parm.port = 0;
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL0, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCALTSEL1, &parm);
   parm.value = 1; /* fsc used as output */
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCDIR, &parm);
   parm.value = 1;
   ioctl(GPIO_PortFd, DANUBE_PORT_IOCOD, &parm);
#endif

   return ret;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy80800_Register(BOARD_t* pBoard)
{
   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pBoard->pszBoardName = BOARD_NAME_EASY80800;
   pBoard->Init = BOARD_Easy80800_Init;
   pBoard->RemoveBoard = BOARD_Easy80800_RemoveBoard;
   return IFX_SUCCESS;
}



