


/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_teridian73M1866.c
   \date 2006-10-01
   \brief Interface implementation for Teridian73M1866 board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_teridian73M1866.h"
#include "common_fxo.h"

#ifdef TERIDIAN_FXO
   #include "teridian_fxo.h"
#endif   

#ifdef DANUBE
   #include "board_easy50712.h"
#elif AR9
   #include "board_easy508xx.h"
#elif VINAX
   #include "board_easy80800.h"
#else
#endif

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** Max Teridian channels on VMMC */
enum { MAX_TERIDIAN_CH_VMMC = 1 };

/** Name of board */
static const char* BOARD_NAME_TERIDIAN73M1866 = "Teridian73M1866";

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Teridian73M1866_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Teridian73M1866_PrepareNames(BOARD_t* pBoard);
IFX_void_t BOARD_Teridian73M1866_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Teridian73M1866_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Teridian73M1866_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_FXO_t* pFXODevice = IFX_NULL;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First three variables (id, type, sub type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   pBoard->nUseSys_FD = IFX_FALSE;

   /* allocate memory for FXO device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for FXO Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* Init structure for FXO device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_FXO;
   pFXODevice = &pDevice->uDevice.stFxo;
   pFXODevice->nFxoType = FXO_TYPE_TERIDIAN;
   pFXODevice->nIrqNum = INT_NUM_IRQ;
   pFXODevice->nCountry = TERIDIAN_COUNTRY_TER;
#ifdef DANUBE
   pFXODevice->Reset = BOARD_Easy50712_Reset;
   pFXODevice->SetPin_Direction_onGPIO = BOARD_Easy50712_SetPin_Direction_onGPIO;
#elif AR9
   pFXODevice->Reset = BOARD_Easy508XX_Reset;
   pFXODevice->SetPin_Direction_onGPIO = BOARD_Easy508XX_SetPin_Direction_onGPIO;
#elif VINAX
   pFXODevice->Reset = BOARD_Easy80800_Reset;
   pFXODevice->SetPin_Direction_onGPIO = BOARD_Easy80800_SetPin_Direction_onGPIO;
#else
#endif 
   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      free(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, FXO device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

#ifdef TERIDIAN_FXO
   if (TERIDIAN_Register(pFXODevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, FXO device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#endif

   if (BOARD_Teridian73M1866_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   pBoard->nMaxFxoCh = MAX_TERIDIAN_CH_VMMC;

   pBoard->rgoFXOs = calloc(pBoard->nMaxFxoCh, sizeof(FXO_t));
   if (pBoard->rgoFXOs == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory to prepare board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* TERIDIAN */
   pBoard->nFxoCh_FD = calloc(pBoard->nMaxFxoCh, sizeof(IFX_int32_t));

   if (pBoard->nFxoCh_FD == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory to FXO channels. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxFxoCh; i++)
   {
       pBoard->nFxoCh_FD[i] = -1;
   }

   /* Store chip count 
      It is necessary to set it. Otherwise EVENT_Check() doesn't detect events */
   pBoard->nChipCnt = 1;
   
   return IFX_SUCCESS;
}

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_return_t BOARD_Teridian73M1866_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#ifdef FXO
   /** Teridian device name without numbers, like /dev/dus */
   strncpy(pBoard->pszFxoChDevName, sTERIDIAN_CH_DEV_NAME, 
           strlen(sTERIDIAN_CH_DEV_NAME));

   /** Device control name numbers, like /dev/dus[board id]0, ... */
   strncpy(pBoard->pszFxoCtrlDevName, sTERIDIAN_CTRL_DEV_NAME, 
           strlen(sTERIDIAN_CTRL_DEV_NAME));

   /* get device number depending on board type */
#if (defined(DANUBE) && !defined(EASY50712_V3))
   /* for DANUBE(EASY50712_V2) extension board uses first device */
   snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);
#else /* (defined(DANUBE) && !defined(EASY50712_V3)) */
   /* for DANUBE(EASY50712_V3) and EASY508XX extension board uses second device,
      first device is used on main board */
   snprintf(tmp_buf, 2, "%d", (int) pBoard->nID + 1);
#endif /* (defined(DANUBE) && !defined(EASY50712_V3)) */

   if (DEV_NAME_LEN > (strlen(tmp_buf) + strlen(pBoard->pszFxoCtrlDevName)))
   {
      strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Buffer overflow, array index of 'pszFxoCtrlDevName' "
            "may be outside the bounds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   snprintf(tmp_buf, 2, "0");
   /* sprintf(tmp_buf, "0"); */
   if (DEV_NAME_LEN > (strlen(tmp_buf) + strlen(pBoard->pszFxoCtrlDevName)))
   {
      strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Buffer overflow, array index of 'pszFxoCtrlDevName' "
            "may be outside the bounds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   } 
#endif /* FXO */
   return IFX_SUCCESS;
}

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Teridian73M1866_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      free(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      free(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->rgoFXOs != IFX_NULL)
   {
      free(pBoard->rgoFXOs);
      pBoard->rgoFXOs = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      free(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      free(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->nFxoCh_FD != IFX_NULL)
   {
      free(pBoard->nFxoCh_FD);
      pBoard->nFxoCh_FD = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      free(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Init board.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Teridian73M1866_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
    /* check input arguments */
   if ((pBoard == IFX_NULL) || (pProgramArg == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Teridian73M1866_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
        == IFX_ERROR)
   {
       return IFX_ERROR;
   }
#ifdef FXO
   /* FXO */
   if (pProgramArg->oArgFlags.nFXO)
   {
      if (IFX_SUCCESS != Common_Set_Fxo_FDs(pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error setting FXO fds. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         Common_Close_Fxo_FDs(pBoard);
         return IFX_ERROR;
      }

      if (IFX_SUCCESS != FXO_Init(pBoard, GPIO_TERIDIAN_EXT, FXO_TYPE_TERIDIAN))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error FXO initialization. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         Common_Close_Fxo_FDs(pBoard);
         return IFX_ERROR;
      }
   }
#endif /* FXO */

   return IFX_SUCCESS;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Teridian73M1866_Register(BOARD_t* pBoard)
{
   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pBoard->pszBoardName = BOARD_NAME_TERIDIAN73M1866;
   pBoard->Init = BOARD_Teridian73M1866_Init;
   pBoard->RemoveBoard = BOARD_Teridian73M1866_RemoveBoard;
   return IFX_SUCCESS;
}




