

/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file device_danube.c
   \date 2006-10-01
   \brief Interface implementation for vinetic vip device.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy334.h"
#ifdef HAVE_DRV_VINETIC_HEADERS
   #include "drv_vinetic_errno.h"
#else
    #error "drv_vinetic is missing"
#endif

#ifndef USE_FILESYSTEM
/** Default FW file includes, to be provided */

#include "vin-vip_bbd_fxs.c"
#include "vin-vip_pDRAMfw.c"
#include "vin-vip_pPRAMfw.c"
#endif /* USE_FILESYSTEM */

#include "device_vinetic_vip.h"


/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/** Names of access modes */
static const IFX_char_t* VIN_ACCESS_MODES[] =
{
   "VIN_ACCESS_SPI",
   "VIN_ACCESS_SCI",
   "VIN_ACCESS_PAR_16BIT",
   "VIN_ACCESS_PAR_8BIT",
   "VIN_ACCESS_PARINTEL_MUX16",
   "VIN_ACCESS_PARINTEL_MUX8",
   "VIN_ACCESS_PARINTEL_DMUX8",
   "VIN_ACCESS_PARINTEL_DMUX8_BE",
   "VIN_ACCESS_PARINTEL_DMUX8_LE",
   "VIN_ACCESS_PAR_8BIT_V2",
   IFX_NULL
};

/* ============================= */
/* Global variable definition    */
/* ============================= */


#ifdef USE_FILESYSTEM
/** Prepare file names for VIP */
IFX_char_t* sPRAMFile_VIP = "voice_vin-vip_firmware.bin";
IFX_char_t* sPRAMFile_VIP_Old = "pramfw.bin";
IFX_char_t* sDRAMFile_VIP = "dramfw.bin";
/** File holding coefficients. */
IFX_char_t* sBBD_CRAM_File_VIP = "vin-vip_bbd_fxs.bin";
#endif /* USE_FILESYSTEM */

/** Device names */
IFX_char_t* sCH_DEV_NAME_VIP = "/dev/vin";
IFX_char_t* sCTRL_DEV_NAME_VIP = "/dev/vin";
IFX_char_t* sSYS_DEV_NAME_VIP = "/dev/easy334/";

/** Holding device dependent data */
VINETIC_IO_INIT VineticDevInitStruct;


/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t DEVICE_VINETIC_VIP_Fillin_FW_Ptr(IFX_void_t* pDrvInit, 
                                              BOARD_t* pBoard,
                                              IFX_char_t* pPath);
IFX_void_t DEVICE_VINETIC_VIP_Release_FW_Ptr(IFX_void_t* pDrvInit);
IFX_return_t DEVICE_VINETIC_VIP_InitSystem(BOARD_t* pBoard);
IFX_return_t DEVICE_VINETIC_VIP_ResetSystem(IFX_int32_t dev_num, 
                                            BOARD_t* pBoard);
IFX_return_t DEVICE_VINETIC_VIP_InitSilicon(BOARD_t* pBoard, IFX_char_t* pPath);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Fills in firmware pointers.

   \param pDrvInit - handle to VINETIC_IO_INIT structure, not nil
   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR

   \remarks
     In case no firmware binaries is found in the file system or no file system
     is used, default binaries will be used
*/
IFX_return_t DEVICE_VINETIC_VIP_Fillin_FW_Ptr(IFX_void_t* pDrvInit, 
                                              BOARD_t* pBoard,
                                              IFX_char_t* pPath)
{
   VINETIC_IO_INIT* vinit = (VINETIC_IO_INIT *) pDrvInit;
   
#ifdef USE_FILESYSTEM
   IFX_uint8_t* pPram = IFX_NULL;
   IFX_uint8_t* pDram = IFX_NULL;
   IFX_uint8_t* pBBD = IFX_NULL;
   IFX_char_t pszFullFilename[MAX_PATH_LEN] = {0};
   IFX_boolean_t bFoundBinFile = IFX_FALSE;
#endif /* USE_FILESYSTEM */


   if ((vinit == IFX_NULL) || (pBoard == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Set all entries to nil */
   memset(vinit, 0, sizeof(VINETIC_IO_INIT));

#ifdef USE_FILESYSTEM
   /* Reading PRAM_FW file */
   /* Find PRAM_FW file with default names */
   /* First check for new name, next for old one */
   bFoundBinFile = Common_FindPRAM_FW(pCpuDevice, pPath, pszFullFilename);
   if(IFX_TRUE == bFoundBinFile)
   {
      /* Fill in PRAM_FW buffers from binary */
      ret = Common_ReadBinFile(pszFullFilename,
                               &pPram, (IFX_int32_t *) &vinit->pram_size);
      /* Release PRAM_FW memory if applicable */
      if ((IFX_ERROR == ret) || (IFX_NULL == pPram))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error reading firmware file %s.\n(File: %s, line: %d)\n",
               pszFullFilename, __FILE__, __LINE__));

         if (pPram != IFX_NULL)
         {
            free(pPram);
            pPram = IFX_NULL;
         }
         return IFX_ERROR;
      }
      else
      {
         vinit->pPRAMfw = pPram;
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Filesystem firmware binary used (%s).\n",
               pszFullFilename));
      } /* if ((IFX_ERROR == ret_fwram) ... */
   } /* if(IFX_TRUE == bFileFound) */
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, firmware file %s is not found.\n(File: %s, line: %d)\n",
            pCpuDevice->pszPRAM_FW_File[0], __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Reading DRAM_FW file */
   /* Reset flag */
   bFoundBinFile = IFX_FALSE;
   /* Find DRAM_FW file with default name */
   ret = Common_CreateFullFilename(pCpuDevice->pszDRAM_FW_File, 
                                   pPath, pszFullFilename);
   if (IFX_SUCCESS == ret) 
   {
      ret = Common_CheckFile(pszFullFilename);
      if (IFX_SUCCESS == ret)
      {
         /* Fill in DRAM_FW buffers from binary */
         ret = Common_ReadBinFile(pszFullFilename,
                                  &pDram, (IFX_int32_t *) &vinit->dram_size);

         /* Release DRAM_FW memory if applicable */
         if ((IFX_ERROR == ret) || (IFX_NULL == pDram)) 
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error reading firmware DRAM file %s.\n(File: %s, line: %d)\n",
                  pCpuDevice->pszDRAM_FW_File, __FILE__, __LINE__));

            if (pDram != IFX_NULL)
            {
               free(pDram);
               pDram = IFX_NULL;
            }
            return IFX_ERROR;
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                  ("Filesystem firmware DRAM binary used (%s).\n",
                   pszFullFilename));

            vinit->pDRAMfw = pDram;
            bFoundBinFile = IFX_TRUE;
         }
      }
   }
   if( IFX_TRUE != bFoundBinFile)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, firmware DRAM file %s is not found.\n(File: %s, line: %d)\n",
             pCpuDevice->pszDRAM_FW_File, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Reading BBD_CRAM file */
   /* Reset flag */
   bFoundBinFile = IFX_FALSE;
   /* Find BBD_CRAM file with default name */
   ret = Common_CreateFullFilename(pCpuDevice->pszBBD_CRAM_File, 
                                   pPath, pszFullFilename);
   if (IFX_SUCCESS == ret) 
   {
      ret = Common_CheckFile(pszFullFilename);
      if (IFX_SUCCESS == ret)
      {
         /* Fill in BBD_CRAM buffers from binary */
         ret = Common_ReadBinFile(pszFullFilename,
                                  &pBBD, (IFX_int32_t *) &vinit->bbd_size);

         /* Release BBD_CRAM memory if applicable */
         if ((IFX_ERROR == ret) || (IFX_NULL == pBBD)) 
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error reading BBD file %s.\n(File: %s, line: %d)\n",
                  pCpuDevice->pszBBD_CRAM_File, __FILE__, __LINE__));

            if (pBBD != IFX_NULL)
            {
               free(pBBD);
               pBBD = IFX_NULL;
            }
            return IFX_ERROR;
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, ("Filesystem BBD binary used (%s).\n",
                                             pszFullFilename));
            vinit->pBBDbuf = pBBD;
            bFoundBinFile = IFX_TRUE;
         }
      }
   }
   if( IFX_TRUE != bFoundBinFile)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, BBD file %s is not found.\n(File: %s, line: %d)\n",
             pCpuDevice->pszBBD_CRAM_File, __FILE__, __LINE__));
      return IFX_ERROR;
   }

#else /* USE_FILESYSTEM */

   /* If file system support isn't used, use default PRAM/DRAM/BBD bins */

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("Default firmware PRAM binaries used\n"));
   vinit->pPRAMfw = (IFX_uint8_t *) vinetic_fw_pram_vip;
   vinit->pram_size = sizeof(vinetic_fw_pram_vip);

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("Default firmware DRAM binaries used.\n"));
   vinit->pDRAMfw = (IFX_uint8_t *) vinetic_fw_dram_vip;
   vinit->dram_size = sizeof(vinetic_fw_dram_vip);

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
         ("Default BBD binaries used.\n"));;
   vinit->pBBDbuf = (IFX_uint8_t *) bbd_buf_vip;
   vinit->cram_size = sizeof(bbd_buf_vip);

#endif /* USE_FILESYSTEM */

   return IFX_SUCCESS;
} /* Fillin_FW_Ptr() */


/**
   Release in firmware pointers.

   \param pDrvInit - handle to VINETIC_IO_INIT structure, not nil
   \param pBoard - pointer to board

   \return none
*/
IFX_void_t DEVICE_VINETIC_VIP_Release_FW_Ptr(IFX_void_t* pDrvInit)
{
   VINETIC_IO_INIT* vinit = (VINETIC_IO_INIT *) pDrvInit;
      
   if (vinit == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

#ifdef USE_FILESYSTEM

   /* Pram fw was read from file, as default isn't set: free memory */
   if (vinit->pPRAMfw != IFX_NULL)
   {
      free(vinit->pPRAMfw);
      vinit->pPRAMfw = IFX_NULL;
   }

   /* Dram fw was read from file, as default isn't set: free memory */
   if (vinit->pDRAMfw != IFX_NULL)
   {
      free(vinit->pDRAMfw);
      vinit->pDRAMfw = IFX_NULL;
   }

   if (vinit->pBBDbuf != IFX_NULL)
   {
      free(vinit->pBBDbuf);
      vinit->pBBDbuf = IFX_NULL;
   }

#endif /* USE_FILESYSTEM */

} /* Release_FW_Ptr() */

/**
   Basic device initialization prior to vinetic initialization.

   \param sysAccMode - system access mode (8/16-bit uC Mot/IntMux/IntDemux,
                       8Bit SPI)to set. This access mode must be converted
                       in a vinetic understandable access mode before doing
                       the vinetic driver initialization.
   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      - In case of 8bit SPI, for the CPE system, base address  should be set to
        0x1F if not otherwise programmed by bootstrapping
      - Caller must make sure that the access mode is  supported
*/
IFX_return_t DEVICE_VINETIC_VIP_InitSystem(BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_uint32_t chipnum = 0, i = 0;
   IFX_int32_t nAccessMode;

/** This two variables are VINETIC specific */
   IFX_int32_t vin_access_mode = 0;
   IFX_uint32_t irq_num = 0, base_address = 0;
   VINETIC_BasicDeviceInit_t vin_init;
   EASY334_Config_t config_vip;
   EASY334_Reset_t reset_vip;
   EASY334_GetDevParam_t chip_param_vip;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   nAccessMode = SYSTEM_AM_DEFAULT_VIP;
   memset(&config_vip, 0, sizeof(config_vip));
   config_vip.nAccessMode = sysAccMode;
   config_vip.nClkRate = 0; /* 0 default 2048 KHz */

   /* set system configuration, CPLD acces mode, ... */
   if (IFX_ERROR == ioctl(pBoard->nSystem_FD, FIO_EASY334_CONFIG,
                          (IFX_int32_t) &config_vip))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd FIO_EASY334_CONFIG."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));      
      return IFX_ERROR;
   }
   /* go on with initialization */
   chipnum = ioctl(pBoard->nSystem_FD, FIO_EASY334_INIT, 0);
   if (chipnum < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd FIO_EASY334_INIT."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));      
      return IFX_ERROR;
   }
   
   /* Store chip count */
   pBoard->nChipCnt = chipnum;
   
   /* initialize each vinetic device */
   for (i = 0; i < chipnum; i++)
   {
      ret = IFX_ERROR;
      /* activate chip reset */
      memset(&reset_vip, 0, sizeof(EASY334_Reset_t));
         
      reset_vip.nChipNum = i;
      reset_vip.nResetMode = EASY334_RESET_ACTIVE;
      if (ioctl(pBoard->nSystem_FD, FIO_EASY334_RESETCHIP,
                (IFX_int32_t) &reset_vip) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd FIO_EASY334_RESETCHIP."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));        
         break;
      }
   
      memset(&chip_param_vip, 0, sizeof(EASY334_GetDevParam_t));
            
      /* read vinetic device configuration parameters from board driver */
      if (ioctl(pBoard->nSystem_FD, FIO_EASY334_GETBOARDPARAMS,
          (IFX_int32_t) &chip_param_vip) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd FIO_EASY334_GETBOARDPARAMS."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));      
         break;
      }
      base_address = chip_param_vip.nBaseAddrPhy;
      /* This two are not used */
      /* clk_rate = chip_param_cpe.nClkRate;
      access_mode = chip_param_cpe.nAccessMode; */
      irq_num = chip_param_vip.nIrqNum;
      
      switch (nAccessMode)
      {
         case EASY334_ACCESS_16BIT_MOTOROLA:
         case EASY334_ACCESS_16BIT_INTELDEMUX:
             vin_access_mode = VIN_ACCESS_PAR_16BIT;
         break;
         case EASY334_ACCESS_16BIT_INTELMUX:
            vin_access_mode = VIN_ACCESS_PARINTEL_MUX16;
         break;
         case EASY334_ACCESS_8BIT_MOTOROLA:
            vin_access_mode = VIN_ACCESS_PAR_8BIT;
         break;
         case EASY334_ACCESS_8BIT_INTELMUX:
            vin_access_mode = VIN_ACCESS_PARINTEL_MUX8;
         break;
         case EASY334_ACCESS_8BIT_INTELDEMUX:
            /* complicated case :
                  VINETIC 2.1 : VIN_ACCESS_PARINTEL_DMUX8_LE
                  Vinetic 1.x : VIN_ACCESS_PARINTEL_DMUX8_BE
               Vinetic driver must decide and do appropriate conversion
               when receiving this flag */
            vin_access_mode = VIN_ACCESS_PARINTEL_DMUX8_BE;
         break;
         case EASY334_ACCESS_SPI:
            vin_access_mode = VIN_ACCESS_SPI;
         break;
         default:
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Unknown Physical Access Mode 0x%lX"
                  "(File: %s, line: %d)\n", vin_access_mode,
                  __FILE__, __LINE__));
         break;
      }
   
      /* configure vinetic device  */
         memset(&vin_init, 0, sizeof(VINETIC_BasicDeviceInit_t));
         vin_init.AccessMode = vin_access_mode;
         vin_init.nBaseAddress = base_address;
         vin_init.nIrqNum = irq_num;

      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("Configure VINETIC device with access mode %s, "
             "base address 0x%0X, IRQ 0x%0X\n",
             VIN_ACCESS_MODES[vin_init.AccessMode],
             (int) vin_init.nBaseAddress, vin_init.nIrqNum));
      
      if (ioctl(pBoard->nDevCtrl_FD, FIO_VINETIC_BASICDEV_INIT,
                (IFX_int32_t) &vin_init) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd FIO_VINETIC_BASICDEV_INIT."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));      
         break;
      }
      
      /* clear reset to allow vinetic access */
      memset(&reset_vip, 0, sizeof(EASY334_Reset_t));
         
      reset_vip.nChipNum = i;
      reset_vip.nResetMode = EASY334_RESET_DEACTIVE;
      if (ioctl(pBoard->nSystem_FD, FIO_EASY334_RESETCHIP,
                (IFX_int32_t) &reset_vip) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd FIO_EASY334_RESETCHIP."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));      
         break;
      }
         
      /* this loop processed successfully */
      ret = IFX_SUCCESS;
   } /* for (i = 0; i < chipnum; i++)*/

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERROR: Basic Device Initialization fails on device %d"
            "(File: %s, line: %d)\n",
            i, __FILE__, __LINE__));
   }

   return ret;
} /* DEVICE_VINETIC_VIP_InitSystem() */


/**
   Basic device reset.

   \param dev_num - id of device to reset
   \param pBoard - pointer to board

   \return 0 if successful, otherwise -1.
   \remarks currently this function is not used in tapidemo,
           left in case it would be ever needed
*/
IFX_return_t DEVICE_VINETIC_VIP_ResetSystem(IFX_int32_t dev_num, 
                                            BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   EASY334_Reset_t reset_vip;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Activate chip reset */
   /* Reset internal vinetic device data */
   /* activate chip reset */
   memset(&reset_vip, 0, sizeof(EASY334_Reset_t));
      
   reset_vip.nChipNum = dev_num;
   reset_vip.nResetMode = EASY334_RESET_ACTIVE;
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_RESETCHIP,
               (IFX_int32_t) &reset_vip);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with iocl cmd FIO_EASY334_RESETCHIP."
            "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
   }
      
   /* configure vinetic device  */
   if (ret == IFX_SUCCESS)
   {
      ret = ioctl(pBoard->nDevCtrl_FD, FIO_VINETIC_DEV_RESET, 0);
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with iocl cmd FIO_VINETIC_DEV_RESET."
               "(File: %s, line: %d)\n",
                __FILE__, __LINE__));
      }
   }
      
   if (ret == IFX_SUCCESS)
   {
      /* clear reset to allow vinetic access */
      memset(&reset_vip, 0, sizeof(EASY334_Reset_t));
          
      reset_vip.nChipNum = dev_num;
      reset_vip.nResetMode = EASY334_RESET_DEACTIVE;
      ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_RESETCHIP,
                  (IFX_int32_t) &reset_vip);
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with iocl cmd FIO_EASY334_RESETCHIP."
               "(File: %s, line: %d)\n",
                __FILE__, __LINE__));
      }
   }

   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERROR: Device Reset fails on device %d"
            "(File: %s, line: %d)\n",
            dev_num, __FILE__, __LINE__));
   }

   return ret;
} /* DEVICE_VINETIC_VIP_ResetSystem() */

/**
   Silicon initialization.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download
   
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t DEVICE_VINETIC_VIP_InitSilicon(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_TAPI_CH_INIT_t Init;
   IFX_int32_t i = 0;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   DEVICE_VINETIC_VIP_Fillin_FW_Ptr(pBoard->pDevDependStuff, pBoard, pPath);

   /* Set tapi init structure */
   memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
   Init.nMode = IFX_TAPI_INIT_MODE_VOICE_CODER;
   Init.pProc = (IFX_void_t*) pBoard->pDevDependStuff;

   /* Initialize all tapi channels */
   for (i = 0; i <= pBoard->nMaxCoderCh; i++)
   {
      if (pBoard->nMaxCoderCh == i)
      {
         /* Read version */
         DEVICE_VINETIC_VIP_GetVersions(pBoard);
         break;
      }
      
      /* Initialize all system channels */
      if (IFX_SUCCESS != ioctl(pBoard->nCh_FD[i], IFX_TAPI_CH_INIT, 
                               (IFX_int32_t) &Init))
      {
         break;
      }

      /* Set appropriate feeding on all (analog) line channels */
      if (i < pBoard->nMaxAnalogCh)
      {
         /* Set line in standby */
         if (IFX_SUCCESS != ioctl(pBoard->nCh_FD[i], IFX_TAPI_LINE_FEED_SET,
                                  IFX_TAPI_LINE_FEED_STANDBY))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Err set line to standby for ch %d, (File: %s, line: %d)\n",
                  (int) i, __FILE__, __LINE__));
            break;
         }
      }
   } /* for */

   DEVICE_VINETIC_VIP_Release_FW_Ptr(pBoard->pDevDependStuff);

   if (pBoard->nMaxCoderCh != i)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Init failed for ch %d, (File: %s, line: %d)\n",
            (int) i, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
} /* BVINETIC_InitSilicon() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Read version after succesfull initialization and display it.

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
   
   \param pBoard - pointer to board

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t DEVICE_VINETIC_VIP_GetVersions(BOARD_t* pBoard)
{
   IFX_return_t ret;
   VINETIC_IO_VERSION vinVers;
   IFX_uint8_t* pVersionNumber;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&vinVers, 0, sizeof(VINETIC_IO_VERSION));

   ret = ioctl(pBoard->nDevCtrl_FD, FIO_VINETIC_VERS, (IFX_int32_t) &vinVers);
   if (IFX_SUCCESS == ret)
   {
      TAPIDEMO_PRINTF(("VINETIC [version 0x%2X, type 0x%2X, channels %d] ready!\n"
                       "FW Version %d.%d\n",
                       vinVers.nChip, vinVers.nType, vinVers.nChannel,
                       vinVers.nEdspVers, vinVers.nEdspIntern));

      pVersionNumber = (IFX_uint8_t*) &vinVers.nDrvVers;
      TAPIDEMO_PRINTF(("Driver version %d.%d.%d.%d\n",
                       pVersionNumber[0],
                       pVersionNumber[1],
                       pVersionNumber[2],
                       pVersionNumber[3]));
#ifdef ABACUS_TEST
      ABACUS_GET_FW_VERSION(pBoard->pCtrl, vinVers);
#endif
   }
   else
   {
      TAPIDEMO_PRINTF(("VINETIC version could not be read.\n"));
   }

   return ret;
} /* DEVICE_VINETIC_VIP_GetVersions() */

/**
   Device initialization.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCES on ok, otherwise IFX_ERROR

   \remarks
     In case no firmware binaries is found in the file system or no file system
     is used, default binaries will be used
*/
IFX_return_t DEVICE_VINETIC_VIP_Init(BOARD_t* pBoard, IFX_char_t* pPath)
{
   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (DEVICE_VINETIC_VIP_InitSystem(pBoard) == IFX_ERROR)
       return IFX_ERROR;

   if (DEVICE_VINETIC_VIP_InitSilicon(pBoard, pPath) == IFX_ERROR)
       return IFX_ERROR;
   
   return IFX_SUCCESS;
}

/**
   Handles the DRIVER error codes and stack.

   \param nCode - error code

   \remarks
*/
IFX_void_t DEVICE_VINETIC_VIP_DecodeErrnoForDrv(IFX_int32_t nCode)
{
#if 0
   /* Not supported in VINETIC at the moment. */
   IFX_int32_t i = 0;


   for (i = 0; i < VINETIC_ERRNO_CNT; ++i)
   {
      if (VINETIC_drvErrnos[i] == (nCode & 0xFFFF))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("LL Err: %s\n", VINETIC_drvErrStrings[i]));
         break;
      }
   }
#endif
}

/**
   Return last error number for vinetic driver.

   \param nFd  -  channel file descriptor

   \return number of error if successful, otherwise IFX_ERROR.
   \remarks
*/
IFX_return_t DEVICE_VINETIC_VIP_GetErrnoNum(IFX_int32_t nFd)
{
   IFX_int32_t nLastErr = 0, ret;

   ret = ioctl(nFd, FIO_VINETIC_LASTERR, (IFX_int32_t) &nLastErr);

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Failed to get error number for fd %d, (File: %s, line: %d)\n",
             nFd, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return nLastErr;

}



