

/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file duslic_fxo.c
   \date 2006-11-28
   \brief This file contain implementation of DuSLIC FXO.

   It initializes DuSLIC (FW, channels, ...) and uses PCM or channel mapping
   and timeslot usage. Basically when connection to DuSLIC is started phone
   channel must be mapped to pcm channel and then pcm connection between this
   pcm channels and pcm channel on DuSLIC is started.
*/
#ifdef LINUX
   #include "tapidemo_config.h"
#endif

#include "ifx_types.h"
#include "tapidemo.h"

#ifdef FXO
#ifdef DUSLIC_FXO

/* ============================= */
/* Includes                      */
/* ============================= */

#ifdef HAVE_DRV_DUSLIC_HEADERS
    #include "drv_duslic.h"
    #include "drv_duslic_io.h"
#else
    #error "drv_duslic is missing"
#endif

#ifdef HAVE_DRV_TAPI_HEADERS
    #include "drv_tapi_io.h"
#else
    #error "drv_tapi is missing"
#endif
#include "danube_bbd_fxo.c"
#include "pcm.h"
#include "voip.h"

/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Global variable definition    */
/* ============================= */

/** Device names */
IFX_char_t* sDUSLIC_CH_DEV_NAME = "/dev/dus";
IFX_char_t* sDUSLIC_CTRL_DEV_NAME = "/dev/dus";

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */

static IFX_return_t DUSLIC_GetLastErr(BOARD_t* pBoard);
IFX_return_t DUSLIC_InitChannel(BOARD_t* pBoard, IFX_int32_t nChNum);
IFX_return_t DUSLIC_GetVersions(BOARD_t* pBoard);

/* ============================= */
/* Local function definition     */
/* ============================= */

/**
   Configure PCM interface for Duslic device, ...

   \param pBoard  - pointer to board

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark 
*/
IFX_return_t DUSLIC_CfgPCMInterface(BOARD_t* pBoard)
{

   IFX_int32_t fd = -1;
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));

   /* Use PCM clock slave mode */
   pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE;

   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
        ("Duslic device - Configure PCM interface as slave, %s kHz\n",
         TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));
   
   ret = ioctl(fd, IFX_TAPI_PCM_IF_CFG_SET, (IFX_int32_t) &pcm_if);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Err configure PCM interface "
            "(File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* set PCM frequency - only for main board*/
   /* oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq; */

   return IFX_SUCCESS;
} /* DUSLIC_CfgPCMInterface() */

/**
   Initialize DUSLIC channels

   \param pBoard - handle to board
   \param nChNum - duslic channel to initialize

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t DUSLIC_InitChannel(BOARD_t* pBoard, IFX_int32_t nChNum)
{
   IFX_uint32_t ret = IFX_SUCCESS;
   DUS_IO_INIT_t dusIoInit;
   IFX_TAPI_CH_INIT_t dusTapiChInit;
   IFX_int32_t fd = -1;
   IFX_TAPI_LINE_TYPE_CFG_t line_type;


   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevOfCh(pBoard, nChNum);

   memset(&dusIoInit, 0, sizeof(DUS_IO_INIT_t));
   memset(&dusTapiChInit, 0, sizeof(IFX_TAPI_CH_INIT_t));

   dusIoInit.cram.ac_coef        = ac_coef;
   dusIoInit.cram.dc_coef        = dc_coef;
   dusIoInit.cram.gen_coef       = gen_coef;
   dusIoInit.cram.gen_coef_size  = 0x6;
   dusIoInit.cram.nFormat        = DUS_IO_CRAM_FORMAT_15;

   dusTapiChInit.nMode = IFX_TAPI_INIT_MODE_PCM_PHONE;
   dusTapiChInit.pProc = &dusIoInit;
   ret = ioctl(fd, IFX_TAPI_CH_INIT, &dusTapiChInit);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("IFX_TAPI_CH_INIT failed for ch %d. (File: %s, line: %d)\n",
            (int) nChNum, __FILE__, __LINE__));
      DUSLIC_GetLastErr(pBoard);
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
        ("IFX_TAPI_CH_INIT ok for ch %d.\n",(int) nChNum));

   memset(&line_type, 0, sizeof(IFX_TAPI_LINE_TYPE_CFG_t));
   line_type.lineType = IFX_TAPI_LINE_TYPE_FXO;
   line_type.nDaaCh = 0;
   /* Set line type */
   ret = ioctl(fd, IFX_TAPI_LINE_TYPE_SET, &line_type);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("IFX_TAPI_LINE_TYPE_SET with IFX_TAPI_LINE_TYPE_FXO failed."
            " (File: %s, line: %d)\n", __FILE__, __LINE__));
      DUSLIC_GetLastErr(pBoard);
      return (IFX_ERROR);
   }

   return ret;
}


/**
   Initialize DUSLIC device.

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t DUSLIC_Init(BOARD_t* pBoard, GPIO_ACCESS_DEV_t eType)
{
   DUS_BasicDeviceInit_t dusInit;
   IFX_uint32_t ret = IFX_SUCCESS;
   IFX_int32_t fd = -1;
   IFX_int32_t i = 0, nDrvDbgLevel;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_FXO_t *pFxoDevice = IFX_NULL;

   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pDevice = Common_GetDevice_FXO(pBoard->pDevice, FXO_TYPE_DUSLIC);
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("DuSLIC FXO device not found."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pFxoDevice = &pDevice->uDevice.stFxo;

#if 0 /* karol */
   if (DUSLIC_SetPCM_Master_overGPIO() != IFX_SUCCESS)
   {
      /* Error set PCM configuration (master, ...) over GPIO */
   } 
#endif

   /* Set pin direction to output */
   if (pFxoDevice->SetPin_Direction_onGPIO(eType) != IFX_SUCCESS)
   {
       /* Error set GPIO port direction to out  */
   }

   /* Set DuSLIC reset */
   if (pFxoDevice->Reset(pBoard, (IFX_int32_t)eType, IFX_TRUE) != IFX_SUCCESS)
   {
       /* Error set chip reset */
   }

   memset(&dusInit, 0, sizeof(DUS_BasicDeviceInit_t));

   fd = Common_GetFxoDevCtrlFd(pBoard);

   dusInit.AccessMode = DUS_ACCESS_SPI;
   dusInit.nIrqNum = pFxoDevice->nIrqNum;

   ret = ioctl(fd, FIO_DUS_BASICDEV_INIT, &dusInit);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ioctl FIO_DUS_BASICDEV_INIT returned an error. "
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      DUSLIC_GetLastErr(pBoard);
      return IFX_ERROR;
   }
   /* Clear DuSLIC reset */
   if (pFxoDevice->Reset(pBoard, (IFX_int32_t)eType, IFX_FALSE) != IFX_SUCCESS)
   {
       /* Error clear chip reset */
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
        ("ioctl FIO_DUS_BASICDEV_INIT ok.\n"));

   /* check debug level */
   if (DBG_LEVEL_NORMAL > pBoard->pCtrl->pProgramArg->nDbgLevel)
   {
      /* set normal debug level for driver */
      nDrvDbgLevel = 0x02;
   }
   else
   {
      /* set high debug level for driver */
      nDrvDbgLevel = 0x03;
   }

   ret = ioctl(fd, FIO_DUS_REPORT_SET, nDrvDbgLevel);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ioctl FIO_DUS_REPORT_SET returned an error. "
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      DUSLIC_GetLastErr(pBoard);
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
        ("ioctl FIO_DUS_REPORT_SET ok.\n"));

   if (DUSLIC_CfgPCMInterface(pBoard) == IFX_ERROR)
   {
      return IFX_ERROR;
   }

   /* Initialize all DUSLIC channels */
   for (i = 0; i <= pBoard->nMaxFxoCh; i++)
   {
      if (pBoard->nMaxFxoCh == i)
      {
         /* Read version */
         DUSLIC_GetVersions(pBoard);
         break;
      }

      /* Init Duslic channel. */
      if (DUSLIC_InitChannel(pBoard, i) != IFX_SUCCESS)
      {
         ret = IFX_ERROR;
         break;
      }
   }
   if (IFX_ERROR != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("DuSLIC FXO initialized.\n"));
   }

   return ret;
}


/**
   Display DUSLIC version

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/

IFX_return_t DUSLIC_GetVersions(BOARD_t* pBoard)
{
   IFX_return_t ret;
   DUS_IO_Version_t duslicVers;
   IFX_uint8_t* pVersionNumber;
   IFX_int32_t fd;


   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   memset(&duslicVers, 0, sizeof(DUS_IO_Version_t));

   ret = ioctl(fd, FIO_DUS_VERS, (IFX_int32_t) &duslicVers);
   if (IFX_SUCCESS == ret)
   {
      TAPIDEMO_PRINTF(
         ("DUSLIC [version 0x%02X, type 0x%02X, channels %d] ready!\n",
          duslicVers.nChip, duslicVers.nType, duslicVers.nChannel));

      pVersionNumber = (IFX_uint8_t*) &duslicVers.nDrvVers;
      TAPIDEMO_PRINTF(("DRV Version %d.%d.%d.%d",
                       pVersionNumber[0],
                       pVersionNumber[1],
                       pVersionNumber[2],
                       pVersionNumber[3]));

      pVersionNumber = (IFX_uint8_t*) &duslicVers.nTapiVers;
      TAPIDEMO_PRINTF((" and included TAPI version %d.%d.%d.%d\n",
                       pVersionNumber[0],
                       pVersionNumber[1],
                       pVersionNumber[2],
                       pVersionNumber[3]));
   }
   else
   {
      TAPIDEMO_PRINTF(("DUSLIC version could not be read.\n"));
   }

   return ret;
}


/**
   Display DUSLIC last error.

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
static IFX_return_t DUSLIC_GetLastErr(BOARD_t* pBoard)
{
   IFX_return_t ret;
   IFX_int32_t nDuslicErr = 0;
   IFX_int32_t fd;


   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   ret = ioctl(fd, FIO_DUS_LASTERR, (IFX_int32_t) &nDuslicErr);
   if (IFX_SUCCESS == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("DUSLIC last error is: 0x%0X\n", (int) nDuslicErr));
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Could not get DUSLIC error. "
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
   }

   return ret;
}

/* ============================= */
/* Global function definition   */
/* ============================= */

/**
   Register device to the board.

   \param pFxoDevice - pointer to device
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t DUSLIC_Register(TAPIDEMO_DEVICE_FXO_t* pFxoDevice)
{
   /* check input arguments */
   if (pFxoDevice == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pFxoDevice->Version = DUSLIC_GetVersions;
   pFxoDevice->GetLastErr = DUSLIC_GetLastErr;
   return IFX_SUCCESS;
}

#endif /* DUSLIC_FXO */
#endif /* FXO */
/** ---------------------------------------------------------------------- */



