
/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file teridian_fxo.c
   \date 2007-07-13
   \brief This file contain implementation of FXO.

   It initializes 73M1X66 chipset and uses PCM or channel mapping
   and timeslot usage. Basically when connection to 73M1X66 is started phone
   channel must be mapped to pcm channel and then pcm connection between this
   pcm channels and pcm channel on 73M1X66 is started.
*/

#include "ifx_types.h"
#ifdef LINUX
#include "tapidemo_config.h"
#endif

#ifdef FXO
#ifdef TERIDIAN_FXO
/* ============================= */
/* Includes                      */
/* ============================= */

#ifdef HAVE_DRV_TERIDIAN_HEADERS
   #include "73m1966_io.h"
#else
   #error "drv_teridian is missing"
#endif /* HAVE_DRV_TERIDIAN_HEADERS */

#ifdef HAVE_DRV_TAPI_HEADERS
   #include "drv_tapi_io.h"
#else
    #error "drv_tapi is missing"
#endif /* HAVE_DRV_TAPI_HEADERS */

#include "tapidemo.h"
#include "pcm.h"

#include "teridian_fxo.h"

#ifdef USE_FILESYSTEM
/** File holding coefficients. */
IFX_char_t* sBBD_CRAM_File_TER = "ter1x66_bbd_fxo.bin";
#endif /* USE_FILESYSTEM */

/** Device names */
IFX_char_t* sTERIDIAN_CH_DEV_NAME = "/dev/ter";
IFX_char_t* sTERIDIAN_CTRL_DEV_NAME = "/dev/ter";

/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t TERIDIAN_InitChannel(BOARD_t* pBoard, 
                                  TAPIDEMO_DEVICE_FXO_t *pFxoDevice,
                                  IFX_int32_t nChNum);
static IFX_return_t TERIDIAN_GetVersions(BOARD_t* pBoard);
static IFX_return_t TERIDIAN_GetLastErr(BOARD_t* pBoard);
IFX_return_t TERIDIAN_CfgPCMInterface(BOARD_t* pBoard);

/* ============================= */
/* Local function definition     */
/* ============================= */

/**
   Initialize 73M1X66 channels

   \param pBoard - handle to board
   \param pFxoDevice - pointer to Teridian FXO device structure 
   \param nChNum - 73M1X66 channel to initialize

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t TERIDIAN_InitChannel(BOARD_t* pBoard,
                                  TAPIDEMO_DEVICE_FXO_t *pFxoDevice,
                                  IFX_int32_t nChNum)
{
   M1966_CH_INIT_STRUCT_t terInit;
   IFX_TAPI_CH_INIT_t terTapiChInit;
   IFX_TAPI_LINE_TYPE_CFG_t line_type;
   IFX_TAPI_LINE_VOLUME_t terVolume;
   IFX_int32_t fd = -1;
   IFX_uint32_t ret = IFX_SUCCESS;
#ifdef USE_FILESYSTEM
   IFX_uint8_t *pCountryCode = IFX_NULL; 
   IFX_int32_t nSize = 0;
   IFX_char_t pszFullFilename[MAX_PATH_LEN] = {0};
   /* True if required BIN file is found, else FALSE */
   IFX_boolean_t bFoundBinFile = IFX_FALSE;
#endif

   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevOfCh(pBoard, nChNum);
   memset(&terInit, 0, sizeof(M1966_CH_INIT_STRUCT_t));
   memset(&terTapiChInit, 0, sizeof(IFX_TAPI_CH_INIT_t));

#ifdef USE_FILESYSTEM
   /* Find BBD file with default name */
   ret = Common_CreateFullFilename(sBBD_CRAM_File_TER, 
                                   pBoard->pCtrl->pProgramArg->sPathToDwnldFiles, 
                                   pszFullFilename);
   if (IFX_SUCCESS == ret) 
   {
      ret = Common_CheckFile(pszFullFilename);
      if (IFX_SUCCESS == ret)
      {
         ret = Common_ReadBinFile(pszFullFilename,
                                  &pCountryCode, &nSize);

         /* release bbd memory if applicable */
         if ((IFX_ERROR == ret) || (IFX_NULL == pCountryCode)) 
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error reading BBD file %s. Using default country settings.\n",
                  sBBD_CRAM_File_TER));
      
            if (pCountryCode != IFX_NULL)
            {
               free(pCountryCode);
               pCountryCode = IFX_NULL;
            }
            terInit.country_code = pFxoDevice->nCountry;
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                  ("Filesystem BBD binary used for Teridian FXO (%s).\n",
                   pszFullFilename));
            terInit.country_code = atoi(pCountryCode);
         }
         bFoundBinFile = IFX_TRUE;
      }
   }
   if( IFX_TRUE != bFoundBinFile)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("BBD binary %s is not found. Using default country settings.\n",
            sBBD_CRAM_File_TER));
      terInit.country_code = pFxoDevice->nCountry;   
   }
#else
   terInit.country_code = pFxoDevice->nCountry;
#endif

   terTapiChInit.nMode = IFX_TAPI_INIT_MODE_PCM_PHONE;
   terTapiChInit.pProc = &terInit;

   ret = ioctl(fd, IFX_TAPI_CH_INIT, &terTapiChInit);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("IFX_TAPI_CH_INIT failed for ch %d, %s. (File: %s, line: %d)\n",
            (int) nChNum, pBoard->pszBoardName, __FILE__, __LINE__));
      TERIDIAN_GetLastErr(pBoard);
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("%s IFX_TAPI_CH_INIT ok for ch %d.\n",
          pBoard->pszBoardName, (int) nChNum));

   memset(&line_type, 0, sizeof(IFX_TAPI_LINE_TYPE_CFG_t));
   line_type.lineType = IFX_TAPI_LINE_TYPE_FXO;
   line_type.nDaaCh = 0;
   /* Set line type */
   ret = ioctl(fd, IFX_TAPI_LINE_TYPE_SET, &line_type);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("IFX_TAPI_LINE_TYPE_SET with IFX_TAPI_LINE_TYPE_FXO failed, %s."
            " (File: %s, line: %d)\n", 
            pBoard->pszBoardName, __FILE__, __LINE__));
      TERIDIAN_GetLastErr(pBoard);
      return (IFX_ERROR);
   }

   memset(&terVolume, 0, sizeof(IFX_TAPI_LINE_VOLUME_t));
   terVolume.nGainRx = 0;
   terVolume.nGainTx = 0;
   /* Set volume */
   ret = ioctl(fd, IFX_TAPI_PHONE_VOLUME_SET, &terVolume);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("IFX_TAPI_PHONE_VOLUME_SET with IFX_TAPI_LINE_TYPE_FXO failed, %s."
             " (File: %s, line: %d)\n", 
             pBoard->pszBoardName, __FILE__, __LINE__));
      TERIDIAN_GetLastErr(pBoard);
      return (IFX_ERROR);
   }

 #ifdef USE_FILESYSTEM
   free(pCountryCode);
 #endif

   return IFX_SUCCESS;
}

/**
   Configure PCM interface for Teridian device, ...

   \param pBoard  - pointer to board

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark 
*/
IFX_return_t TERIDIAN_CfgPCMInterface(BOARD_t* pBoard)
{

   IFX_int32_t fd = -1;
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));

   /* Use PCM clock slave mode */
   pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE;

   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
        ("Teridian device - Configure PCM interface as slave, %s kHz\n",
         TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));
   
   ret = ioctl(fd, IFX_TAPI_PCM_IF_CFG_SET, (IFX_int32_t) &pcm_if);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Err configure PCM interface "
            "(File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* set PCM frequency - only for main board*/
   /* oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq; */

   return IFX_SUCCESS;
} /* TERIDIAN_CfgPCMInterface() */

/**
   Initialize 73M1X66 chip set.

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t TERIDIAN_Init(BOARD_t* pBoard, GPIO_ACCESS_DEV_t eType)
{
   IFX_uint32_t            ret = IFX_SUCCESS;
   IFX_int32_t             i = 0, nDrvDbgLevel;
   IFX_int32_t             fd = -1;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_FXO_t *pFxoDevice = IFX_NULL;

   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Wrong input arguments."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pDevice = Common_GetDevice_FXO(pBoard->pDevice, FXO_TYPE_TERIDIAN);
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("TERIDIAN FXO device not found."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pFxoDevice = &pDevice->uDevice.stFxo;

#if 0 /*karol*/
   if (TERIDIAN_SetPCM_Master_overGPIO() != IFX_SUCCESS)
   {
      /* Error set PCM configuration (master, ...) over GPIO */
   } 
#endif

   /* Set pin direction to output */
   if (pFxoDevice->SetPin_Direction_onGPIO(eType) != IFX_SUCCESS)
   {
      /* Error set GPIO port direction to out  */
   }

   /* Set reset */
   if (pFxoDevice->Reset(pBoard, (IFX_int32_t)eType, IFX_TRUE) != IFX_SUCCESS)
   {
      /* Error set chip reset */
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   /* todo: basic device initialization */

   /* Clear reset */
   if (pFxoDevice->Reset(pBoard, (IFX_int32_t)eType, IFX_FALSE) != IFX_SUCCESS)
   {
      /* Error clear chip reset */
   }

   /* check debug level */
   if (DBG_LEVEL_NORMAL > pBoard->pCtrl->pProgramArg->nDbgLevel)
   {
      /* set normal debug level for driver */
      nDrvDbgLevel = 0x02;
   }
   else
   {
      /* set high debug level for driver */
      nDrvDbgLevel = 0x03;
   }

   /* todo: Report set ? */
   ret = ioctl(fd, IFX_TAPI_DEBUG_REPORT_SET, nDrvDbgLevel);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ioctl IFX_TAPI_DEBUG_REPORT_SET returned an error. "
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      TERIDIAN_GetLastErr(pBoard);
      return IFX_ERROR;
   }

   if (TERIDIAN_CfgPCMInterface(pBoard) == IFX_ERROR)
   {
      return IFX_ERROR;
   }

   /* Initialize all 73M1X66 channels */
   for (i = 0; i <= pBoard->nMaxFxoCh; i++)
   {
      if (pBoard->nMaxFxoCh == i)
      {
         /* Read version */
         TERIDIAN_GetVersions(pBoard);
         break;
      }

      /* Init Daa channel. */
      if (TERIDIAN_InitChannel(pBoard, pFxoDevice, i) != IFX_SUCCESS)
      {
         ret = IFX_ERROR;
         break;
      }
   }
   if (IFX_ERROR != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("Teridian FXO initialized.\n"));
   }

   return ret;
}


/**
   Display 73M1X66 version

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/

static IFX_return_t TERIDIAN_GetVersions(BOARD_t* pBoard)
{
   IFX_int32_t fd = -1;
   IFX_char_t Version[80];
   IFX_return_t ret = IFX_SUCCESS;

   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);
   ret = ioctl(fd, IFX_TAPI_VERSION_GET, (IFX_char_t*) &Version[0]);
   if (ret == IFX_SUCCESS) 
   {
      TAPIDEMO_PRINTF(("Teridian Version: %s\n" , Version));
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Teridian version could not be read."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      TERIDIAN_GetLastErr(pBoard);
   }
   /* printf("Teridian 73M1966 - Version 0.98\n"); */

   return IFX_SUCCESS;
}

/**
   Display Teridian last error.

   \param pBoard - handle to board

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
static IFX_return_t TERIDIAN_GetLastErr(BOARD_t* pBoard)
{
   IFX_TAPI_Error_t Error = {-1};
   IFX_int32_t fd = -1;
#if 0
   IFX_int32_t i = 0;
#endif


   if (pBoard == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Wrong input arguments."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   fd = Common_GetFxoDevCtrlFd(pBoard);

   ioctl(fd, IFX_TAPI_LASTERR, (IFX_int32_t)&Error);
   if (Error.nCode != -1)
   {
      /* we have additional information */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error Code 0x%X occured\n", Error.nCode));
#if 0
      for (i = 0; i < ERRNO_CNT; ++i)
      {
         if (drvErrnos[i] == (Error.nCode & 0xffff))
         {
             TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                   ("%s\n", drvErrStrings[i]));
         }
      }
      for (i = 0; i < Error.nCnt; ++i)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("%s:%d Code 0x%X\n", Error.stack[i].sFile,
               Error.stack[i].nLine, Error.stack[i].nCode));
      }
#endif
   }

   return IFX_SUCCESS;
}

/* ============================= */
/* Global function definition   */
/* ============================= */
   
/**
   Register device to the board.

   \param pFxoDevice - pointer to device
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/

IFX_return_t TERIDIAN_Register(TAPIDEMO_DEVICE_FXO_t* pFxoDevice)
{
   /* check input arguments */
   if (pFxoDevice == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pFxoDevice->Version = TERIDIAN_GetVersions;
   pFxoDevice->GetLastErr = TERIDIAN_GetLastErr;
   return IFX_SUCCESS;
}

#endif /* TERIDIAN_FXO */
#endif /* FXO */

/** ---------------------------------------------------------------------- */



